      SUBROUTINE RWUPDT(N,R,LDR,W,B,ALPHA,COS,SIN)
      INTEGER N,LDR
      REAL ALPHA
      REAL R(LDR,N),W(N),B(N),COS(N),SIN(N)
C     **********
C
C     SUBROUTINE RWUPDT
C
C     GIVEN AN N BY N UPPER TRIANGULAR MATRIX R, THIS SUBROUTINE
C     COMPUTES THE QR DECOMPOSITION OF THE MATRIX FORMED WHEN A ROW
C     IS ADDED TO R. IF THE ROW IS SPECIFIED BY THE VECTOR W, THEN
C     RWUPDT DETERMINES AN ORTHOGONAL MATRIX Q SUCH THAT WHEN THE
C     N+1 BY N MATRIX COMPOSED OF R AUGMENTED BY W IS PREMULTIPLIED
C     BY (Q TRANSPOSE), THE RESULTING MATRIX IS UPPER TRAPEZOIDAL.
C     THE MATRIX (Q TRANSPOSE) IS THE PRODUCT OF N TRANSFORMATIONS
C
C           G(N)*G(N-1)* ... *G(1)
C
C     WHERE G(I) IS A GIVENS ROTATION IN THE (I,N+1) PLANE WHICH
C     ELIMINATES ELEMENTS IN THE (N+1)-ST PLANE. RWUPDT ALSO
C     COMPUTES THE PRODUCT (Q TRANSPOSE)*C WHERE C IS THE
C     (N+1)-VECTOR (B,ALPHA). Q ITSELF IS NOT ACCUMULATED, RATHER
C     THE INFORMATION TO RECOVER THE G ROTATIONS IS SUPPLIED.
C
C     THE SUBROUTINE STATEMENT IS
C
C       SUBROUTINE RWUPDT(N,R,LDR,W,B,ALPHA,COS,SIN)
C
C     WHERE
C
C       N IS A POSITIVE INTEGER INPUT VARIABLE SET TO THE ORDER OF R.
C
C       R IS AN N BY N ARRAY. ON INPUT THE UPPER TRIANGULAR PART OF
C         R MUST CONTAIN THE MATRIX TO BE UPDATED. ON OUTPUT R
C         CONTAINS THE UPDATED TRIANGULAR MATRIX.
C
C       LDR IS A POSITIVE INTEGER INPUT VARIABLE NOT LESS THAN N
C         WHICH SPECIFIES THE LEADING DIMENSION OF THE ARRAY R.
C
C       W IS AN INPUT ARRAY OF LENGTH N WHICH MUST CONTAIN THE ROW
C         VECTOR TO BE ADDED TO R.
C
C       B IS AN ARRAY OF LENGTH N. ON INPUT B MUST CONTAIN THE
C         FIRST N ELEMENTS OF THE VECTOR C. ON OUTPUT B CONTAINS
C         THE FIRST N ELEMENTS OF THE VECTOR (Q TRANSPOSE)*C.
C
C       ALPHA IS A VARIABLE. ON INPUT ALPHA MUST CONTAIN THE
C         (N+1)-ST ELEMENT OF THE VECTOR C. ON OUTPUT ALPHA CONTAINS
C         THE (N+1)-ST ELEMENT OF THE VECTOR (Q TRANSPOSE)*C.
C
C       COS IS AN OUTPUT ARRAY OF LENGTH N WHICH CONTAINS THE
C         COSINES OF THE TRANSFORMING GIVENS ROTATIONS.
C
C       SIN IS AN OUTPUT ARRAY OF LENGTH N WHICH CONTAINS THE
C         SINES OF THE TRANSFORMING GIVENS ROTATIONS.
C
C     SUBPROGRAMS CALLED
C
C       FORTRAN-SUPPLIED ... ABS,SQRT
C
C     ARGONNE NATIONAL LABORATORY. MINPACK PROJECT. MARCH 1980.
C     BURTON S. GARBOW, DUDLEY V. GOETSCHEL, KENNETH E. HILLSTROM,
C     JORGE J. MORE
C
C     **********
      INTEGER I,J,JM1
      REAL COTAN,ONE,P5,P25,ROWJ,TAN,TEMP,ZERO
      DATA ONE,P5,P25,ZERO /1.0E0,5.0E-1,2.5E-1,0.0E0/
C
      DO 60 J = 1, N
         ROWJ = W(J)
         JM1 = J - 1
C
C        APPLY THE PREVIOUS TRANSFORMATIONS TO
C        R(I,J), I=1,2,...,J-1, AND TO W(J).
C
         IF (JM1 .LT. 1) GO TO 20
         DO 10 I = 1, JM1
            TEMP = COS(I)*R(I,J) + SIN(I)*ROWJ
            ROWJ = -SIN(I)*R(I,J) + COS(I)*ROWJ
            R(I,J) = TEMP
   10       CONTINUE
   20    CONTINUE
C
C        DETERMINE A GIVENS ROTATION WHICH ELIMINATES W(J).
C
         COS(J) = ONE
         SIN(J) = ZERO
         IF (ROWJ .EQ. ZERO) GO TO 50
         IF (ABS(R(J,J)) .GE. ABS(ROWJ)) GO TO 30
            COTAN = R(J,J)/ROWJ
            SIN(J) = P5/SQRT(P25+P25*COTAN**2)
            COS(J) = SIN(J)*COTAN
            GO TO 40
   30    CONTINUE
            TAN = ROWJ/R(J,J)
            COS(J) = P5/SQRT(P25+P25*TAN**2)
            SIN(J) = COS(J)*TAN
   40    CONTINUE
C
C        APPLY THE CURRENT TRANSFORMATION TO R(J,J), B(J), AND ALPHA.
C
         R(J,J) = COS(J)*R(J,J) + SIN(J)*ROWJ
         TEMP = COS(J)*B(J) + SIN(J)*ALPHA
         ALPHA = -SIN(J)*B(J) + COS(J)*ALPHA
         B(J) = TEMP
   50    CONTINUE
   60    CONTINUE
      RETURN
C
C     LAST CARD OF SUBROUTINE RWUPDT.
C
      END
